@@ -105,7 +105,7 @@ def unique_pairs(listOfItems:list):
             yield x,y
 
 
-def find_matching_blocks(block1:Block,block2:Block,tol:float=5E-5):  
+def find_matching_blocks(block1:Block,block2:Block,tol:float=1E-6):  
     """Takes two blocks and finds all matching pairs
 
     Args:
@@ -254,7 +254,7 @@ def get_face_intersection(face1:Face,face2:Face,block1:Block,block2:Block,tol:fl
                     match_location.append({"i1":I1[0],"j1":p+J1[0],"k1":q+K1[0],'i2':p2+I2[0],'j2':J2[0],'k2':q2+K2[0]})
                 if K2[0]==K2[1]:
                     match_location.append({"i1":I1[0],"j1":p+J1[0],"k1":q+K1[0],'i2':p2+I2[0],'j2':q2+J2[0],'k2':K2[0]})
-        df = df.append(match_location,ignore_index=True)
+        df = pd.concat([df, pd.DataFrame(match_location)], ignore_index=True)
 
     elif J1[0] == J1[1]: # J is constant in face 1 
         combo = product(range(0,X1.shape[0]), range(0,X1.shape[1]))
@@ -274,7 +274,7 @@ def get_face_intersection(face1:Face,face2:Face,block1:Block,block2:Block,tol:fl
                     match_location.append({"i1":p+I1[0],"j1":J1[0],"k1":q+K1[0],'i2':p2+I2[0],'j2':J2[0],'k2':q2+K2[0]})
                 if K2[0]==K2[1]:
                     match_location.append({"i1":p+I1[0],"j1":J1[0],"k1":q+K1[0],'i2':p2+I2[0],'j2':q2+J2[0],'k2':K2[0]})
-            df = df.append(match_location,ignore_index=True)
+            df = pd.concat([df, pd.DataFrame(match_location)], ignore_index=True)
 
     elif K1[0] == K1[1]: # K is constant in face 1 
         combo = product(range(0,X1.shape[0]), range(0,X1.shape[1]))
@@ -294,7 +294,7 @@ def get_face_intersection(face1:Face,face2:Face,block1:Block,block2:Block,tol:fl
                     match_location.append({"i1":p+I1[0],"j1":q+J1[0],"k1":K1[0],'i2':p2+I2[0],'j2':J2[0],'k2':q2+K2[0]})
                 if K2[0]==K2[1]:
                     match_location.append({"i1":p+I1[0],"j1":q+J1[0],"k1":K1[0],'i2':p2+I2[0],'j2':q2+J2[0],'k2':K2[0]})
-        df = df.append(match_location,ignore_index=True)
+        df = pd.concat([df, pd.DataFrame(match_location)], ignore_index=True)
     
     # Checking for split faces 
     if len(df)>4:
@@ -476,8 +476,8 @@ def connectivity_fast(blocks:List[Block]):
     # Find the gcd of all the blocks 
     for block_indx in range(len(blocks)):
         block = blocks[block_indx]
-        gcd_array.append(math.gcd(block.IMAX-1, math.gcd(block.JMAX-1, block.KMAX-1)))
-    gcd_to_use = min(gcd_array)
+        gcd_array.append(math.gcd(block.IMAX-1, math.gcd(block.JMAX-1, block.KMAX-1)))    
+    gcd_to_use = min(gcd_array) # You need to use the minimum gcd otherwise 1 block may not exactly match the next block. They all have to be scaled the same way.
     new_blocks = reduce_blocks(deepcopy(blocks),gcd_to_use)
 
     # Find Connectivity 
@@ -521,7 +521,7 @@ def connectivity(blocks:List[Block]):
     outer_faces = list()      
     face_matches = list()
     matches_to_remove = list()
-    # df_matches, blocki_outerfaces, blockj_outerfaces = find_matching_blocks(blocks[1],blocks[2])    # This function finds partial matches between blocks
+    # df_matches, blocki_outerfaces, blockj_outerfaces = find_matching_blocks(blocks[4],blocks[7],1E-12)    # This function finds partial matches between blocks
 
     combos = combinations_of_nearest_blocks(blocks) # Find the 6 nearest Blocks and search through all that. 
     t = trange(len(combos))
@@ -548,7 +548,6 @@ def connectivity(blocks:List[Block]):
 
                 temp = face_matches_to_dict(face1,face2,blocks[i],blocks[j])
                 temp['match'] = df
-                # ! Paht check 
                 face_matches.append(temp)
 
         # Update Outer Faces 
